/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#include <iostream>
#include <iomanip>
#include "Attributes.h"
#include "Builder.h"
#include "Kind.h"
#include "Model.h"
#include "StringCount.h"
#include "Style.h"
#include "Type.h"
#include "Wood.h"

bool Attributes::matches(Attributes *other)
{
    AttributesMap::iterator it;

    for (it = attributes.begin(); it != attributes.end(); it++)
    {
        Key key = it->first;

        // Are the corresponding attribute values equal?
        if (! (other->has_key(key) && equals(other, key)) ) return false;
    }

    return true;  // the values all match
}

bool Attributes::has_key(Key key)
{
    return attributes.find(key) != attributes.end();
}

bool Attributes::equals(Attributes *other, const Key key)
{
    boost::any  this_value = get_value(key);
    boost::any other_value = other->get_value(key);

    switch (key)
    {
        case Key::BUILDER:      return equal_builders(this_value, other_value);
        case Key::MODEL:        return equal_models(this_value, other_value);
        case Key::STRING_COUNT: return equal_string_count(this_value, other_value);
        case Key::STYLE:        return equal_styles(this_value, other_value);
        case Key::TYPE:         return equal_types(this_value, other_value);
        case Key::TOP_WOOD:     return equal_woods(this_value, other_value);
        case Key::BACK_WOOD:    return equal_woods(this_value, other_value);
        default:                return false;
    }
}

void Attributes::print()
{
    AttributesMap::iterator it;

    // Loop over the attribute values.
    for (it = attributes.begin(); it != attributes.end(); it++)
    {
        Key        key   = it->first;
        boost::any value = it->second;

        cout << setw(12) << key << ": ";

        // Print each value.
        switch (key)
        {
            case Key::BUILDER:      cout << to_builder(value);      break;
            case Key::MODEL:        cout << to_model(value);        break;
            case Key::STRING_COUNT: cout << to_string_count(value); break;
            case Key::STYLE:        cout << to_style(value);        break;
            case Key::TYPE:         cout << to_type(value);         break;
            case Key::TOP_WOOD:     cout << to_wood(value);         break;
            case Key::BACK_WOOD:    cout << to_wood(value);         break;
        }

        cout << endl;
    }
}

ostream& operator <<(ostream& ostr, const Key attr)
{
    switch (attr)
    {
        case Key::BUILDER:   ostr << "Builder";     break;
        case Key::MODEL:     ostr << "Model";       break;
        case Key::STRING_COUNT:   ostr << "No. strings"; break;
        case Key::STYLE:     ostr << "Style";       break;
        case Key::TYPE:      ostr << "Type";        break;
        case Key::TOP_WOOD:  ostr << "Top wood";    break;
        case Key::BACK_WOOD: ostr << "Back wood";   break;
        default:             ostr << "unspecified";
    }

    return ostr;
}
