/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#ifndef BUILDER_H_
#define BUILDER_H_

#include <iostream>
#include <boost/any.hpp>

using namespace std;

/**
 * The builder attribute values.
 */
enum class Builder
{
    FENDER, MARTIN, GIBSON, COLLINGS, OLSON, RYAN, PRS, ANY,
};

/**
 * Cast a boost::any value to a Builder value.
 * @param value the value to cast.
 * @return the Builder value.
 */
inline Builder to_builder(boost::any value)
{
    return boost::any_cast<Builder>(value);
}

/**
 * Test whether two Builder values are equal.
 * @param value1 the first value.
 * @param value2 the second value.
 * @return true if they're equal, else false.
 */
inline bool equal_builders(boost::any value1, boost::any value2)
{
    return to_builder(value1) == to_builder(value2);
}

/**
 * Overloaded << output operator to print a builder value.
 * @param ostr the output stream.
 * @param builder the builder value.
 * @return the output stream.
 */
inline ostream& operator <<(ostream& ostr, const Builder builder)
{
    switch (builder)
    {
        case Builder::FENDER:   cout << "Fender";   break;
        case Builder::MARTIN:   cout << "Martin";   break;
        case Builder::GIBSON:   cout << "Gibson";   break;
        case Builder::COLLINGS: cout << "Collings"; break;
        case Builder::OLSON:    cout << "Olson";    break;
        case Builder::RYAN:     cout << "Ryan";     break;
        case Builder::PRS :     cout << "PRS";      break;
        default:                cout << "Unspecified";
    }

    return ostr;
}

#endif /* BUILDER_H_ */
