/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#ifndef INSTRUMENT_H_
#define INSTRUMENT_H_

#include <string>
#include "Kind.h"
#include "InstrumentSpec.h"

using namespace std;

/**
 * A musical instrument.
 */
class Instrument
{
public:
    /**
     * Constructor.
     * @param serial_number the instrument's serial number.
     * @param price the instrument's price.
     * @param spec the instrument's specification.
     */
    Instrument(Kind kind, string serial_number, double price,
               InstrumentSpec *spec)
        : kind(kind), serial_number(serial_number), price(price), spec(spec)
    {}

    /**
     * Getter.
     * @return the serial number.
     */
    string get_serial_number() const  { return serial_number; }

    /**
     * Getter.
     * @return the price.
     */
    double get_price() const { return price; }

    /**
     * Setter.
     * @param new_price the new price to set.
     */
    void set_price(float new_price) { price = new_price; }

    /**
     * Getter.
     * @return the specification.
     */
    InstrumentSpec *get_spec() const { return spec; }

    /**
     * Print a matching instrument.
     */
    void print_match();

private:
    Kind kind;
    string serial_number;
    double price;
    InstrumentSpec *spec;
};

inline void Instrument::print_match()
{
    Attributes *attributes = spec->get_attributes();

    cout << "We have a matching " << kind << endl;
    cout << "serial number " << serial_number << endl;
    cout << "for $" << price << endl;

    attributes->print();
    cout << endl;
}

#endif /* INSTRUMENT_H_ */
