/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#ifndef INSTRUMENTSPEC_H_
#define INSTRUMENTSPEC_H_

#include "Attributes.h"

/**
 * An instrument specification.
 */
class InstrumentSpec
{
public:
    /**
     * Constructor.
     * @param attributes the specification's attributes.
     */
    InstrumentSpec(Attributes *attributes) : attributes(attributes) {}

    /**
     * Getter.
     * @return the attributes.
     */
    Attributes *get_attributes() const { return attributes; }

    /**
     * Test whether this specification's attributes
     * matches another specification's attributes.
     * Delegate the matching to Attributes::matches().
     * @param other_spec the other specification.
     * @return true if they match, else false.
     */
    bool matches(InstrumentSpec *other_spec);

private:
    Attributes *attributes;
};

inline bool InstrumentSpec::matches(InstrumentSpec *other_spec)
{
    return attributes->matches(other_spec->get_attributes());
}

#endif /* INSTRUMENTSPEC_H_ */
