/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#include <iostream>
#include <vector>
#include <map>
#include <algorithm>
#include "Inventory.h"

using namespace std;

void Inventory::add_instrument(Kind kind, string serial_number, double price,
                               InstrumentSpec *spec)
{
    Instrument *instrument = new Instrument(kind, serial_number, price, spec);
    inventory.push_back(instrument);
}

Instrument *Inventory::get(string serial_number)
{
    // Loop over the instruments in the inventory.
    for (Instrument *instrument : inventory)
    {
        if (instrument->get_serial_number() == serial_number) return instrument;
    }

    return nullptr;
}

vector<Instrument *> Inventory::search(InstrumentSpec *target_spec)
{
    vector<Instrument *> matching_instruments;

    // Loop over the instruments in the inventory.
    for (Instrument *instrument : inventory)
    {
        InstrumentSpec *instrument_spec = instrument->get_spec();

        // Does the instrument's specification match?
        if (target_spec->matches(instrument_spec))
        {
            matching_instruments.push_back(instrument);
        }
    }

    return matching_instruments;
}
