/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#ifndef INVENTORY_H_
#define INVENTORY_H_

#include <vector>
#include "Instrument.h"
#include "Kind.h"

using namespace std;

/**
 * The inventory of musical instruments.
 */
class Inventory
{
public:
    /**
     * Add an instrument to the inventory.
     * @param serial_number the instrument's serial number.
     * @param price the instrument's price.
     * @param spec the instrument's specification.
     */
    void add_instrument(Kind kind, string serial_number, double price,
                        InstrumentSpec *spec);

    /**
     * Get an instrument with a given serial number.
     * @param serial_number the serial number.
     * @return the instrument.
     */
    Instrument *get(string serial_number);

    /**
     * Search for instruments that match a given specification.
     * @param search_spec the specification to match.
     * @return a vector of the matching instruments.
     */
    vector<Instrument *> search(InstrumentSpec *search_spec);

private:
    vector<Instrument *> inventory;
};

#endif /* INVENTORY_H_ */
