/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#ifndef KIND_H_
#define KIND_H_

#include <iostream>
#include <boost/any.hpp>

using namespace std;

/**
 * The kind of instrument.
 */
enum class Kind
{
    GUITAR, BANJO, DOBRO, FIDDLE, BASS, MANDOLIN,
};

/**
 * Cast a boost::any value to a Kind value.
 * @param value the value to cast.
 * @return the Kind value.
 */
inline Kind to_kind(boost::any value)
{
    return boost::any_cast<Kind>(value);
}

/**
 * Test whether two Kind values are equal.
 * @param value1 the first value.
 * @param value2 the second value.
 * @return true if they're equal, else false.
 */
inline bool equal_kinds(boost::any value1, boost::any value2)
{
    return to_kind(value1) == to_kind(value2);
}

/**
 * Overloaded << output operator to print the kind of instrument.
 * @param ostr the output stream.
 * @param kind the instrument kind.
 * @return the output stream.
 */
inline ostream& operator <<(ostream& ostr, const Kind kind)
{
    switch (kind)
    {
        case Kind::BANJO:    cout << "banjo";    break;
        case Kind::BASS :    cout << "bass";     break;
        case Kind::DOBRO:    cout << "dobro";    break;
        case Kind::FIDDLE:   cout << "fiddle";   break;
        case Kind::GUITAR:   cout << "guitar";   break;
        case Kind::MANDOLIN: cout << "mandolin"; break;
    }

    return ostr;
}

#endif /* KIND_H_ */
