/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#ifndef STYLE_H_
#define STYLE_H_

#include <iostream>
#include <boost/any.hpp>

using namespace std;

/**
 * The style attribute values.
 */
enum class Style
{
    A, F,
};

/**
 * Cast a boost::any value to a Style value.
 * @param value the value to cast.
 * @return the Style value.
 */
inline Style to_style(boost::any value)
{
    return boost::any_cast<Style>(value);
}

/**
 * Test whether two Style values are equal.
 * @param value1 the first value.
 * @param value2 the second value.
 * @return true if they're equal, else false.
 */
inline bool equal_styles(boost::any value1, boost::any value2)
{
    return to_style(value1) == to_style(value2);
}

/**
 * Overloaded << output operator to print a style value.
 * @param ostr the output stream.
 * @param style the style value.
 * @return the output stream.
 */
inline ostream& operator <<(ostream& ostr, const Style style)
{
    switch (style)
    {
        case Style::A: cout << "A"; break;
        case Style::F: cout << "F"; break;
        default:       cout << "unspecified";
    }

    return ostr;
}

#endif /* STYLE_H_ */
