/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#ifndef WOOD_H_
#define WOOD_H_

#include <iostream>
#include <boost/any.hpp>

using namespace std;

enum class Wood
{
    INDIAN_ROSEWOOD, BRAZILIAN_ROSEWOOD, MAHOGANY, MAPLE,
    COCOBOLO, CEDAR, ADIRONDACK, ALDER, SITKA
};

/**
 * Cast a boost::any value to a Wood value.
 * @param value the value to cast.
 * @return the Wood value.
 */
inline Wood to_wood(boost::any value)
{
    return boost::any_cast<Wood>(value);
}

/**
 * Test whether two Wood values are equal.
 * @param value1 the first value.
 * @param value2 the second value.
 * @return true if they're equal, else false.
 */
inline bool equal_woods(boost::any value1, boost::any value2)
{
    return to_wood(value1) == to_wood(value2);
}

inline ostream& operator <<(ostream& ostr, const Wood wood)
{
    switch (wood)
    {
        case Wood::INDIAN_ROSEWOOD:    cout << "Indian Rosewood";     break;
        case Wood::BRAZILIAN_ROSEWOOD: cout << "Brazilian Rosewood";  break;
        case Wood::MAHOGANY:           cout << "Mahogany";            break;
        case Wood::MAPLE:              cout << "Maple";               break;
        case Wood::COCOBOLO:           cout << "Cocobolo";            break;
        case Wood::CEDAR:              cout << "Cedar";               break;
        case Wood::ADIRONDACK:         cout << "Adirondack";          break;
        case Wood::ALDER:              cout << "Alder";               break;
        case Wood::SITKA:              cout << "Sitka";               break;
        default:                       cout << "unspecified";
    }

    return ostr;
}

#endif /* WOOD_H_ */
